<?php
/**
 * Various functions to perform actions in a multi-network environment
 * @package WordPress
 * @subpackage MultiNetwork
 * @version 0.2
 */

if( !function_exists( 'is_multinetwork' ) ) {
	/**
	 * Checks to see if this is a multi-network environment
	 *
	 * Checks to see if the Networks+, WP MultiNetwork or Networks for WordPress plugin
	 * 		is installed and active
	 * @return bool whether or not this is multi-network
	 * @since 0.1
	 * 
	 * @uses WP_PLUGIN_DIR
	 * @uses WPMU_PLUGIN_DIR
	 * @uses $wpdb
	 */
	function is_multinetwork() {
		if( function_exists( 'wpmn_switch_to_network' ) || function_exists( 'switch_to_site' ) || function_exists( 'ra_network_page' ) )
			return true;
			
		if( !file_exists( WP_PLUGIN_DIR . '/wordpress-multi-network/wordpress-multi-network.php' ) && !file_exists( WPMU_PLUGIN_DIR . '/wordpress-multi-network.php' ) && !file_exists( WP_PLUGIN_DIR . '/networks-for-wordpress/index.php' ) && !file_exists( WPMU_PLUGIN_DIR . '/networks-for-wordpress.php' ) && !file_exists( WP_PLUGIN_DIR . '/Networks-Plus/ra-networks.php' ) && !file_exists( WPMU_PLUGIN_DIR . '/ra-network.php' ) )
			return false;
		
		global $wpdb;
		$plugins = $wpdb->get_results( $wpdb->prepare( "SELECT meta_value FROM " . $wpdb->sitemeta . " WHERE meta_key = 'active_sitewide_plugins'" ) );
		foreach( $plugins as $plugin ) {
			if( array_key_exists( 'wordpress-multi-network/wordpress-multi-network.php', maybe_unserialize( $plugin->meta_value ) ) ) {
				require_once( WP_PLUGIN_DIR . '/wordpress-multi-network/wordpress-multi-network.php' );
				return true;
			} elseif( array_key_exists( 'networks-for-wordpress/index.php', maybe_unserialize( $plugin->meta_value ) ) ) {
				require_once( WP_PLUGIN_DIR . '/networks-for-wordpress/index.php' );
				return true;
			} elseif( array_key_exists( 'Networks-Plus/ra-networks.php', maybe_unserialize( $plugin->meta_value ) ) ) {
				require_once( WP_PLUGIN_DIR . '/Networks-Plus/ra-networks.php' );
				return true;
			}
		}
		$sites = $wpdb->get_results( $wpdb->prepare( "SELECT blog_id FROM " . $wpdb->blogs ) );
		foreach( $sites as $site ) {
			$oldblog = $wpdb->set_blog_id( $site->blog_id );
			$plugins = $wpdb->get_results( $wpdb->prepare( "SELECT option_value FROM " . $wpdb->options . " WHERE option_name = 'active_plugins'" ) );
			foreach( $plugins as $plugin ) {
				if( array_key_exists( 'wordpress-multi-network/wordpress-multi-network.php', maybe_unserialize( $plugin->option_value ) ) ) {
					require_once( WP_PLUGIN_DIR . '/wordpress-multi-network/wordpress-multi-network.php' );
					return true;
				} elseif( array_key_exists( 'networks-for-wordpress/index.php', maybe_unserialize( $plugin->option_value ) ) ) {
					require_once( WP_PLUGIN_DIR . '/networks-for-wordpress/index.php' );
					return true;
				} elseif( array_key_exists( 'Networks-Plus/ra-networks.php', maybe_unserialize( $plugin->option_value ) ) ) {
					require_once( WP_PLUGIN_DIR . '/Networks-Plus/ra-networks.php' );
					return true;
				}
			}
			$wpdb->set_blog_id( $oldblog );
		}
		
		return false;
	}
}

if( !function_exists( 'is_plugin_active_for_network' ) ) {
	/**
	 * Check to see if a plug-in is network activated
	 * @uses maybe_unserialize()
	 * @uses get_site_option()
	 */
	/*function is_plugin_active_for_network( $plugin_file ) {
		if( !is_array( $asp = maybe_unserialize( get_site_option( 'active_sitewide_plugins' ) ) ) )
			return false;
		return ( array_key_exists( $plugin_file, $asp ) );
	}*/ /* is_plugin_network_active() function */
	require_once( ABSPATH . '/wp-admin/includes/plugin.php' );
}

if( !function_exists( 'multinetwork_activate' ) ) {
	/**
	 * Network-activate a plugin across all networks in a multi-network installation
	 * 
	 * @param string $file the plugin to be deactivated
	 * @param string $redirect the location to which to redirect once we're finished
	 * @param bool $network_wide whether this plugin should be network-activated or not (should
	 *		always be true)
	 * @param bool $silent whether to fire the normal deactivation actions/filters
	 * @return bool false if this is not a multi-network environment; true if it is
	 * @since 0.2
	 *
	 * @uses $wpdb to perform various actions on the database
	 * @uses do_action() hooks the pre_mn_deactivate_$plugin action to perform any actions
	 * 		that need to occur prior to deactivation (such as removing settings)
	 *		Sends the site_id and blog_id of the network on which the plugin is being deactivated
	 * @uses do_action() hooks the mn_deactivated_$plugin action to perform any actions that
	 * 		need to occur once the plugin has been deactivated
	 *		Sends the site_id and blog_id of the network on which the plugin is being deactivated
	 */
	function multinetwork_activate( $file, $redirect='', $network_wide=true, $silent=false ) {
		if( !is_multinetwork() )
			return false;
		
		$plugin = plugin_basename( trim( $file ) );
			
		global $wpdb;
		$site_info = $wpdb->get_results( $wpdb->prepare( "SELECT site_id, blog_id FROM $wpdb->blogs WHERE site_id != %d GROUP BY site_id", $GLOBALS['site_id'] ) );
		
		if( !empty( $site_info ) ) {
			foreach( $site_info as $network ) {
				if( !$silent )
					do_action( "pre_mn_activate_$plugin", $network );
				
				$asp = maybe_unserialize( $wpdb->get_var( $wpdb->prepare( "SELECT meta_value FROM {$wpdb->sitemeta} WHERE meta_key=%s AND site_id=%d", 'active_sitewide_plugins', $network->site_id ) ) );
				if( is_array( $asp ) && !array_key_exists( $plugin, $asp ) )
					$asp = array_merge( $asp, array( $plugin => time() ) );
				elseif( empty( $asp ) )
					$asp = array( $plugin => time() );
				
				$wpdb->update( $wpdb->sitemeta, array( 'meta_value' => maybe_serialize( $asp ) ), array( 'meta_key' => 'active_sitewide_plugins' , 'site_id' => $network->site_id ), array( '%s' ), array( '%s', '%d' ) );
				
				if( !$silent )
					do_action( "mn_activated_$plugin", $network );
			}
		}
		
		return true;
	}
}

if( !function_exists( 'multinetwork_deactivate' ) ) {
	/**
	 * Network-deactivate a plugin or multiple plugins across all networks 
	 * in a multi-network installation
	 *
	 * @param array|string $plugins the list of plugins to be deactivated
	 * @param bool $silent whether to fire the normal deactivation actions/filters
	 * @return bool false if this is not a multi-network environment; true if it is
	 * @since 0.2
	 *
	 * @uses $wpdb to perform various actions on the database
	 * @uses do_action() hooks the pre_mn_deactivate_$plugin action to perform any actions
	 * 		that need to occur prior to deactivation (such as removing settings)
	 *		Sends the site_id and blog_id of the network on which the plugin is being deactivated
	 * @uses do_action() hooks the mn_deactivated_$plugin action to perform any actions that
	 * 		need to occur once the plugin has been deactivated
	 *		Sends the site_id and blog_id of the network on which the plugin is being deactivated
	 */
	function multinetwork_deactivate( $plugins, $silent = false ) {
		if( !is_multinetwork() )
			return false;
		
		foreach( (array)$plugins as $plugin ) {
			$plugin = plugin_basename( trim( $plugin ) );
			
			global $wpdb;
			$site_info = $wpdb->get_results( $wpdb->prepare( "SELECT site_id, blog_id FROM $wpdb->blogs WHERE site_id != %d GROUP BY site_id", $GLOBALS['site_id'] ) );
			if( !empty( $site_info ) ) {
				foreach( $site_info as $network ) {
					if( !$silent )
						do_action( "pre_mn_deactivate_{$plugin}", $network );
					
					$asp = maybe_unserialize( $wpdb->get_var( $wpdb->prepare( "SELECT meta_value FROM {$wpdb->sitemeta} WHERE meta_key=%s AND site_id=%d", 'active_sitewide_plugins', $network->site_id ) ) );
					if( is_array( $asp ) && array_key_exists( $plugin, $asp ) )
						unset( $asp[$plugin] );
					
					$wpdb->update( $wpdb->sitemeta, array( 'meta_value' => maybe_serialize( $asp ) ), array( 'meta_key' => 'active_sitewide_plugins' , 'site_id' => $network->site_id ), array( '%s' ), array( '%s', '%d' ) );
					
					if( !$silent )
						do_action( "mn_deactivated_{$plugin}", $network );
				}
			}
		}
		
		return true;
	}
}
?>